#include "PAL.h"

void PALImpl::Init()
{
}

void PALImpl::Release()
{
}

bool PALImpl::openAppSettings()
{
    return true;
}

void PALImpl::saveImageToGallery(const char *name)
{
}

bool PALImpl::checkPermission(int permission)
{
	return true;
}
